/**
 * @file main.c
 * @brief Este aplicativo demonstra a funcao IC e OC do temporizador TPM
 * @author Wu Shin Ting
 * @date 30/10/2020
 *
 */

#include "derivative.h" /* include peripheral declarations */

/**
 * @mascarar em um bit
 */
#define GPIO_PIN(x)  ((1)<<(x)) ///< obtem o bit do pino x

/******************************************************************
 * ROTINAS Latch 74573
 */
/**
 * @brief Inicializa os pinos conectados ao latch 74573 onde estao conectados os 8 leds vermelhos
 */
void GPIO_initLatch74573(){
	SIM_SCGC5 |= SIM_SCGC5_PORTC_MASK ;     ///< Habilita o clock da PORTA C

	// Configura os pinos PORTC[10:0] como GPIO de saida 
	PORTC_PCR0 |= PORT_PCR_MUX(0x1);       ///< D0-D7 dos dados (GPIO)
	PORTC_PCR1 |= PORT_PCR_MUX(0x1);
	PORTC_PCR2 |= PORT_PCR_MUX(0x1);
	PORTC_PCR3 |= PORT_PCR_MUX(0x1);
	PORTC_PCR4 |= PORT_PCR_MUX(0x1);
	PORTC_PCR5 |= PORT_PCR_MUX(0x1);
	PORTC_PCR6 |= PORT_PCR_MUX(0x1);
	PORTC_PCR7 |= PORT_PCR_MUX(0x1);
	
	PORTC_PCR10 |= PORT_PCR_MUX(0x1);       ///< /LE do latch

	GPIOC_PDDR |= GPIO_PDDR_PDD(GPIO_PIN(10) |GPIO_PIN(7) | 
			GPIO_PIN(6) | GPIO_PIN(5) | GPIO_PIN(4) | GPIO_PIN(3) | 
			GPIO_PIN(2) | GPIO_PIN(1) | GPIO_PIN(0));       ///< Configura como saidas
	GPIOC_PDOR &= ~GPIO_PDOR_PDO(GPIO_PIN(10) |GPIO_PIN(7) | 
			GPIO_PIN(6) | GPIO_PIN(5) | GPIO_PIN(4) | GPIO_PIN(3) | 
			GPIO_PIN(2) | GPIO_PIN(1) | GPIO_PIN(0)); ///< Inicializa os pinos com 0
}

/******************************************************************
 * ROTINAS TPM
 */
#define COUNTER_OVF 65535         // valor maximo do TPMx_CNT

/**
 * @brief Inicializar o modulo TPM0 que serve o switch NMI e os leds PTC1-PTC4
 */
void TPM0_init (void) {
	SIM_SCGC6 |= SIM_SCGC6_TPM0_MASK;        // habilitar clock de TPM0
	SIM_SOPT2 |= SIM_SOPT2_TPMSRC(0b01);     // configurar a fonte de clock 
	SIM_SOPT2 &= ~SIM_SOPT2_PLLFLLSEL_MASK;  // PLLFLLSEL=SIM_SOPT2[16]=0 (MCGFLLCLK)		
											 // MCGFLLCLK = 20,97152MHz

	//Inicializar contador TPM0
	TPM0_SC &= ~(TPM_SC_DMA_MASK |           // DMA=TPM2_SC[8]=0 (desabilita DMA)
				TPM_SC_TOIE_MASK |           // TOIE=TPM2_SC[6]=0 (desabilita interrupcao "overflow")
				TPM_SC_CPWMS_MASK); 		 // CPWMS=TPM2_SC[5]=0 (TIC/TOC)
	
	TPM0_MOD &= TPM_MOD_MOD(COUNTER_OVF);    // valor de referencia

	TPM0_CNT |= TPM_CNT_COUNT(0x0);          // fazer um acesso de escrita para resetar a contagem
	
	TPM0_CONF &= ~(TPM_CONF_CROT_MASK |      // CROT=TPM2_CONF[18]=0 (nao recarrega o valor de LPTPM no gatilho)
			TPM_CONF_CSOO_MASK |             // CSOO=TPM2_CONF[17]=0 (continua contagem em overflow)
			TPM_CONF_CSOT_MASK |             // CSOT=TPM2_CONF[16]=0 (inicia contagem quando habilitado)
			TPM_CONF_GTBEEN_MASK |			 // GTBEEN=TPM2_CONF[9]=0 (LPTPM contador como base de tempo)
			TPM_CONF_DOZEEN_MASK);           // DOZEEN=TPM2_CONF[5]=0 (continua contagem em hibernacao)
	TPM0_CONF |= TPM_CONF_DBGMODE(0b11);	 // DBGMODE=TPM2_CONF[7:6]=0b11 (continua contagem em Debug)

	TPM0_SC |= (TPM_SC_TOF_MASK |            // baixar bandeira TOF 
				TPM_SC_PS(0b111) );          // prescaler = 128

	// Inicializar o contador TPM0 
	TPM0_SC |= TPM_SC_CMOD(0x1);			// fonte de clock LPTPM
}

/**
 * @brief Habilitar a interrupcao IRQ17
 * @param[in] priority prioridade de atendimento
 */
void TPM0_enableNVICInterrupt(char priority)
{
    NVIC_ISER |= 1 << (33-16) ;      		// Habilitar IRQ17
    NVIC_ICPR |= 1 << (33-16) ;      		// Limpar as pendencias em solicitacao de interrupcoes
    NVIC_IPR4 |= NVIC_IP_PRI_17(priority << 6); ///< seta prioridade 		
}

/**
 * @brief habilitar a interrupcao Overflow
 */
void TPM0_enableOVFInterrupt(void)
{	
	TPM0_STATUS |= TPM_STATUS_TOF_MASK;     // w1c
	TPM0_SC |= TPM_SC_TOIE_MASK;			// habilita interrupcao por Overflow
	return;
}

/**
 * @brief desabilitar a interrupcao Overflow
 */
void TPM0_disableOVFInterrupt(void)
{	
	TPM0_SC &= ~TPM_SC_TOIE_MASK;		   // desabilita interrupcao
	return;
}

/**
 * @brief check whether status flag of Overflow is on
 * @return 0 (OFF)/1 (ON)
 */
uint32_t TPM0_flagOVFOn(void)
{	
	return (TPM0_STATUS & TPM_STATUS_TOF_MASK);
}

/**
 * @brief fazer acesso de escrita com 1 para baixar a bandeira de Overflow
 */
void TPM0_w1cFlagOVF(void)
{	
	TPM0_STATUS |= TPM_STATUS_TOF_MASK;
	return;
}

/**
 * @brief Inicializar o controle do pino NMI (PTA4) por TPM0_CH1 no modo TIC
 */
void TPM0_initSwitchNMI (void) {
	// Inicializar pino TPM0_CH1 
	SIM_SCGC5 |= SIM_SCGC5_PORTA_MASK;        // Habilitar clock de PORTA
	PORTA_PCR4 &= ~PORT_PCR_MUX(0x7);		  // Resetar o campo MUX
	PORTA_PCR4 |= PORT_PCR_MUX(0x3);		  // TPM0_CH1
	
	//Desabilitar o canal
	TPM0_C1SC &= ~(TPM_CnSC_MSB_MASK |      // MSB=TPM0_C1SC[5]=0
				TPM_CnSC_MSA_MASK |         // MSA=TPM0_C1SC[4]=0
				TPM_CnSC_ELSB_MASK |		// ELSB=TPM0_C1SC[3]=0 
				TPM_CnSC_ELSA_MASK );		// ELSB=TPM0_C1SC[2]=0
	TPM0_C1SC &= ~(TPM_CnSC_CHIE_MASK| 			// desabilitar interrupcao
			TPM_CnSC_DMA_MASK); 	    		// desabilitar DMA
	
	// Habilitar no modo TIC
	TPM0_C1SC |= TPM_CnSC_ELSB_MASK;		// captura na borda de descida
				
	// Habilitar o evento de interrupcao
	TPM0_C1SC |= TPM_CnSC_CHF_MASK;      	// limpar pendencias 
	
	TPM0_C1SC |= TPM_CnSC_CHIE_MASK; 	    // habilitar interrupcao do canal 1
}

/**
 * @brief check whether status flag of Switch NMI is on
 * @return 0 (OFF)/1 (ON)
 */
uint32_t TPM0_flagOnSwitchNMI(void)
{
	return (TPM0_STATUS & TPM_STATUS_CH1F_MASK);
}

/**
 * @brief fazer acesso de escrita com 1 para baixar a bandeira do Switch NMI
 */
void TPM0_w1cFlagSwitchNMI(void)
{	
	TPM0_C1SC |= TPM_CnSC_CHF_MASK;     	// limpar solicitacao da chave     
	return;
}

/**
 * @brief Inicializar o controle do pino PTC4 por TPM0_CH3 no modo TOC
 */
void TPM0_initLedPTC4 (void) {
	
	//Habilitar LE do latch
	GPIOC_PDOR |= (1<<10);
	
	//Desabilitar o canal que serve PTC4
	TPM0_C3SC &= ~(TPM_CnSC_MSB_MASK |      // MSB=TPM0_C3SC[5]=0
				TPM_CnSC_MSA_MASK |         // MSA=TPM0_C3SC[4]=0
				TPM_CnSC_ELSB_MASK |		// ELSB=TPM0_C3SC[3]=0 
				TPM_CnSC_ELSA_MASK );		// ELSB=TPM0_C3SC[2]=0
	
	TPM0_C3SC &= ~TPM_CnSC_CHIE_MASK; 	    // desabilitar interrupcao do canal TPM0_CH3
}

/**
 * @brief chaveiar PTC4 de GPIO para TPM0_CH3
 */
void TPM0_GPIO2TPMLedPTC4 (void) {
	PORTC_PCR4 &= ~PORT_PCR_MUX(0x7);		  
	PORTC_PCR4 |= PORT_PCR_MUX(0x4);		  	
}

/**
 * @brief chaveiar PTC4 de TPM0_CH3 para GPIO
 */
void TPM0_TPM2GPIOLedPTC4 (void) {
	PORTC_PCR4 &= ~PORT_PCR_MUX(0x7);		  
	PORTC_PCR4 |= PORT_PCR_MUX(0x1);		  	
}

/**
 * @brief setar o valor de contagem em que o led PTC4 comeca piscar
 * @param[in] valor valor de comparacao para modeo TOC
 */
void TPM0_setCnVLedPTC4 (unsigned short valor) {
	TPM0_C3V = valor;					// setar o instante de saida para led 		
}

/**
 * @brief piscar led PTC4
 */
void TPM0_toggleLedPTC4 (void) {
	TPM0_C3SC |= (TPM_CnSC_MSA_MASK |
				TPM_CnSC_ELSA_MASK); 		// habilita modo toggle
}

/**
 * @brief desabilitar o canal do led PTC4
 */
void TPM0_disableLedPTC4 (void) {
	TPM0_C3SC &= ~(TPM_CnSC_MSA_MASK |
				TPM_CnSC_ELSA_MASK); 		
}

/**
 * @brief habilitar a interrupcao Overflow
 */
void TPM0_enableLedPTC4Interrupt(void)
{	
	TPM0_C3SC |= TPM_CnSC_CHF_MASK; 	    // baixar bandeira (w1c)
	TPM0_C3SC |= TPM_CnSC_CHIE_MASK; 	    // habilitar interrupcao
	return;
}

/**
 * @brief desabilitar a interrupcao Overflow
 */
void TPM0_disableLedPTC4Interrupt(void)
{	
	TPM0_C3SC &= ~TPM_CnSC_CHIE_MASK; 	    // desabilitar interrupcao
	return;
}

/**
 * @brief fazer acesso de escrita com 1 para baixar a bandeira do Led PTC4
 */
void TPM0_w1cFlagLedPTC4(void)
{	
	TPM0_C3SC |= TPM_CnSC_CHF_MASK;     
	return;
}

/**
 * @brief check whether status flag of Led PTC4 is on
 * @return 0 (OFF)/1 (ON)
 */
uint32_t TPM0_flagOnLedPTC4(void)
{
	return (TPM0_STATUS & TPM_STATUS_CH3F_MASK);
}

/**
 * @brief Rotina de servico
 */
void FTM0_IRQHandler () {
	static unsigned short counter;
	
	if (TPM0_flagOnSwitchNMI()) {
		TPM0_setCnVLedPTC4 (TPM0_C1V);
		TPM0_enableOVFInterrupt();		
		counter = 0;							//  contador de controle do intervalo
		TPM0_w1cFlagSwitchNMI();
	} else if (TPM0_flagOnLedPTC4()) {
		TPM0_GPIO2TPMLedPTC4();
		TPM0_w1cFlagLedPTC4();
	} else if (TPM0_flagOVFOn()) {
		counter++;
		if (counter == 5) {
			// Led apagado: 2.5 ciclos completos = 1s --> 5 ciclos = 2s
			TPM0_toggleLedPTC4 ();
			TPM0_enableLedPTC4Interrupt();		
		} else if (counter == 10) {	
			// Led piscante por mais 5 ciclos completos (2s)
			TPM0_disableLedPTC4();
			TPM0_TPM2GPIOLedPTC4 ();
			TPM0_disableLedPTC4Interrupt();					
			TPM0_disableOVFInterrupt();
		}
		TPM0_w1cFlagOVF();
	}

}

/**
 * @brief Controle do intervalo de tempo do led vermelho piscante: 2s apos 
 * 	acionamento da chave NMI iniciam as piscadas e duram as piscadas por 2s. 
 * @return 1 somente para satisfazer a sintaxe C
 */

int main( void)
{		
	// Inicilizar modulo TPM0
	TPM0_init ();
	
	// Inicializar NVIC para IRQ 17
	TPM0_enableNVICInterrupt(2);
	
	// Inicializar NMI com modo IC
	TPM0_initSwitchNMI ();
	
	// Resetar leds vermelhos
	GPIO_initLatch74573();

	// Inicializar o led PTC4 para servir o canal TPM0_CH3
	TPM0_initLedPTC4 ();
	
	for(;;) 
	{
	}
	return 1;
}

